# Generated by Django 5.0.3 on 2024-03-06 10:50

import django_cloud_storage.storage
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ecommerce', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='seo_image_cropped',
            field=models.ImageField(blank=True, editable=False, null=True, storage=django_cloud_storage.storage.CloudStorage, upload_to=''),
        ),
        migrations.AddField(
            model_name='product',
            name='seo_image_cropped',
            field=models.ImageField(blank=True, editable=False, null=True, storage=django_cloud_storage.storage.CloudStorage, upload_to=''),
        ),
        migrations.AddField(
            model_name='productimage',
            name='cropped',
            field=models.ImageField(blank=True, editable=False, null=True, storage=django_cloud_storage.storage.CloudStorage, upload_to=''),
        ),
    ]
