# Generated by Django on 2024-03-06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "ecommerce",
            "0002_category_seo_image_cropped_product_seo_image_cropped_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="discount",
            name="last_activation_run",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="discount",
            name="last_deactivation_run",
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
